/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package edu.umn.d.cs5541.GraphProblem.NQueens;

import edu.umn.d.cs5541.GraphProblem.ProblemState;
import java.util.LinkedList;
import java.util.List;

/**
 *
 * @author rmaclin
 */
public class NQueensProblemState implements ProblemState {

    /**
     * Number of queens needed
     */
    public int N = 0;

    /**
     * Number of queens set
     */
    public int numColumnsSet = 0;

    /**
     * Locations of the queens
     */
    public int[] stateNumbers = null;

    /**
     * Actual cost to this state
     */
    private double actualCost = 0;

    /**
     * Estimated cost from this state
     */
    private double estimatedCost = 0;

    /**
     * Sets up N Queens problem state
     * @param theN
     * @throws Throwable
     */
    public NQueensProblemState(int theN) throws Throwable {
        if (theN <= 0) {
            throw new Throwable("Cannot create an NQueens problem of size " + theN + "!\n");
        }
        N = theN;
        numColumnsSet = 0;
        stateNumbers = new int[N + 1];
    }

    /**
     * Is this state legal?
     * @return
     */
    private boolean legal() {
        for (int i = 1; i <= numColumnsSet; i++) {
            for (int j = i + 1; j <= numColumnsSet; j++) {
                if (stateNumbers[i] == stateNumbers[j]) {
                    return false;
                }
                if (Math.abs(i - j) == Math.abs(stateNumbers[i] - stateNumbers[j])) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * Returns the set of successorStates from this state
     * @return
     * @throws Throwable
     */
    public List<ProblemState> successorStates() throws Throwable {
        if (numColumnsSet < N) {
            List<ProblemState> resultStates = new LinkedList<ProblemState>();
            for (int i = 1; i <= N; i++) {
                ProblemState resultState = (ProblemState) this.clone();
                ((NQueensProblemState) resultState).numColumnsSet++;
                ((NQueensProblemState) resultState).stateNumbers[((NQueensProblemState) resultState).numColumnsSet] = i;
                if (((NQueensProblemState) resultState).legal()) {
                    resultStates.add(resultState);
                }
            }
            return resultStates;
        } else {
            return null;
        }
    }

    /**
     * Is this state a goal?
     * @return
     */
    public boolean isaGoal() {
        return ((numColumnsSet == N) && legal());
    }

    /**
     * Print out this goal to the console
     */
    public void showState() {
        System.out.println("State: ");
        for (int i = 1; i <= N; i++) {
            System.out.print("  " + i + " |");
            for (int j = 1; j <= N; j++) {
                if (stateNumbers[j] == i) {
                    System.out.print("Q|");
                } else {
                    System.out.print(" |");
                }
            }
            System.out.println();
        }
    }

    /**
     * Compare this state to another returning true if equal (in terms of the
     * N Queens state it represents)
     * @param compareState
     * @return
     */
    public boolean isEqualTo(ProblemState compareState) {
        NQueensProblemState theCompareState = (NQueensProblemState) compareState;
        if (numColumnsSet != theCompareState.numColumnsSet) {
            return false;
        }
        for (int i = 1; i <= numColumnsSet; i++) {
            if (stateNumbers[i] != theCompareState.stateNumbers[i]) {
                return false;
            }
        }
        return true;
    }

    /**
     * Make a deep copy of this state
     * @return
     */
    @Override
    public ProblemState clone() {
        NQueensProblemState newState = null;
        try {
            newState = new NQueensProblemState(N);
        }
        catch (Throwable t) {
            return newState;
        }

        newState.numColumnsSet = numColumnsSet;
        for (int i = 1; i <= numColumnsSet; i++) {
            newState.stateNumbers[i] = stateNumbers[i];
        }

        return (ProblemState) newState;
    }

    public void setActualCost(double theCost) {
        actualCost = theCost;
    }

    public double getActualCost() {
        return actualCost;
    }


    public void setEstimatedCost(double theCost) {
        estimatedCost = theCost;
    }

    public double getEstimatedCost() {
        return estimatedCost;
    }
}
