/**
 * Interface to be implemented for a graph problem.  The only required function
 * startState which returns the initial state.  Note that you may want to
 * include initialization routines to set up your problems.
 */

package edu.umn.d.cs5541.GraphProblem;

/**
 * Problem is the interface to be implemented for graph problems.
 */
public interface Problem {

    /**
     * startState returns the initial state of this problem
     * @return an object of type ProblemState
     * @throws Throwable
     */
    public ProblemState startState() throws Throwable;

    /**
     * Returns cost of going from state1 to state2
     *
     * @param state1 starting state
     * @param state2 end state
     * @return actual cost of going from state1 to state2
     */
    public double stateToStateCost(ProblemState state1, ProblemState state2);

    /**
     * Returns ESTIMATED cost of going from state to the goal
     * @param state start state
     * @return cost of going from state to goal
     */
    public double costToGoal(ProblemState state);
}
