/*
 * ProblemState is the interface to be implemented for states of graph Problem
 * objects.
 */

package edu.umn.d.cs5541.GraphProblem;

import java.util.List;

/**
 * Representation of individual states in the graph problem
 */
public interface ProblemState {
    /**
     * Calculates a list of the states reachable from this state
     * @return list of ProblemStates that follow this state
     * @throws Throwable
     */
    public List<ProblemState> successorStates() throws Throwable;

    /**
     * Determines if this state is a goal state, false otherwise
     * @return boolean indicating if goal
     * @throws Throwable
     */
    public boolean isaGoal() throws Throwable;

    /**
     * Routine prints a representation of the this state
     */
    public void showState();

    /**
     * Determines if this state is equal to the passed state
     * @param compareState state to compare to
     * @return true if this state is equivalent to compareState
     */
    public boolean isEqualTo(ProblemState compareState);

    /**
     * Override standard clone routine to make a deep clone of this state
     * @return copy of this state
     */
    public ProblemState clone();

    /**
     * Set the actual cost of making it to this state
     * @param theCost
     */
    public void setActualCost(double theCost);

    /**
     * Get the actual cost of making it to this state
     * @return
     */
    public double getActualCost();

    /**
     * Set the estimated cost of going from here to the(a) goal
     * @param theCost
     */
    public void setEstimatedCost(double theCost);

    /**
     * Get the estimated cost of going from here the(a) goal
     * @return
     */
    public double getEstimatedCost();
}
